"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const lodash_1 = require("lodash");
const util_1 = require("../util");
class Validator {
    constructor(errors = {}) {
        this.processing = false;
        this.successful = false;
        this.errors = errors;
    }
    add(attribute, message) {
        if (this.missed(attribute)) {
            this.errors[attribute] = [];
        }
        if (!this.errors[attribute].includes(message)) {
            this.errors[attribute].push(message);
        }
    }
    has(field) {
        if ((0, lodash_1.isArray)(field)) {
            return (0, util_1.is)(Object.keys(this.errors), field);
        }
        let hasError = (0, lodash_1.has)(this.errors, field);
        if (!hasError) {
            const errors = Object.keys(this.errors).filter((e) => e.startsWith(`${field}.`) || e.startsWith(`${field}[`));
            hasError = errors.length > 0;
        }
        return hasError;
    }
    first(field) {
        if (Array.isArray(field)) {
            for (let i = 0; i < field.length; i++) {
                if (!(0, lodash_1.has)(this.errors, field[i])) {
                    continue;
                }
                return this.first(field[i]);
            }
        }
        const value = this.get(field);
        if (Array.isArray(value))
            return value[0];
        return value; // return it if object like
    }
    firstBy(obj, field) {
        let value;
        if (!field) {
            value = obj[Object.keys(obj)[0]];
        }
        else {
            value = obj[field];
        }
        if (Array.isArray(value))
            value = value[0];
        return value;
    }
    missed(field) {
        return !this.has(field);
    }
    nullState(field) {
        return this.has(field) ? this.missed(field) : null;
    }
    any(fields = [], returnObject) {
        if (returnObject) {
            const errors = {};
            if (!fields.length) {
                return {};
            }
            fields.forEach((key) => (errors[key] = this.get(key)));
            return errors;
        }
        if (!fields.length) {
            return Object.keys(this.errors).length > 0;
        }
        const errors = {};
        fields.forEach((key) => (errors[key] = this.get(key)));
        return Object.keys(errors).length > 0;
    }
    get(field) {
        return (0, lodash_1.get)(this.errors, field) || [];
    }
    all() {
        return this.errors;
    }
    count() {
        return Object.keys(this.errors).length;
    }
    fill(errors) {
        this.errors = errors;
    }
    flush() {
        this.errors = {};
    }
    clear(attribute) {
        if (!attribute) {
            return this.flush();
        }
        const errors = (0, lodash_1.omit)((0, lodash_1.cloneDeep)(this.errors), attribute);
        this.fill(errors);
    }
    isValid() {
        return !this.any();
    }
    onKeydown(event, prefix) {
        const { name } = event.target;
        if (!name)
            return;
        const names = prefix ? [name, `${prefix}.${name}`] : [name];
        this.clear(names);
    }
}
exports.default = new Validator();
